/*      This file is Copyright (C) GVE Software - Geert Vernaeve.
 *      You may use it for non-commercial purposes only.
 *      Distribution is freely permitted, as long as no fee is charged.
 *
 *      For comments, mail me at geert.vernaeve@rug.ac.be
 *                      or geert@einstein.rug.ac.be
 *
 *              CGI SUPPORT MODULE V1.1d
 *
 *      HISTORY:
 *      V1.1b: Fixed some bugs from the V1.1 release.
 *      V1.1c: Added documentation in the source code.
 *      V1.1d:
 *
 *         _agd#######.      ,d#########
 *       _J############.    d###########
 *      .###0"      "###.   ####
 *      d###         "###.  ####
 *      ####          "###. ##########
 *      ####           "###.##########
 *      Q###    ####    "#######
 *      "###b   ####     "######
 *       `Q########D      "#############
 *          "0###0"        "############
 */
#include <stdio.h>
#include <stdlib.h>
#include "cgisup.h"

/* We maintain two linked lists of arguments, i.e. items
 * of the type "value=data".
 * The first is ArgForm and contains all things from a HTML form
 * (e.g. <INPUT NAME=Name> or <SELECT NAME=name>)
 * and the second list, ArgURL, contains all things from the URL
 * of our page (i.e. http://.../cgiwrap?value1=name1&value2=name2&...).
 */
struct Argument {
        struct Argument *Next;
        char *Name;
        char *Data;
};

static struct Argument *ArgForm = NULL, *ArgURL = NULL;

/* Scan the list for a certain Name and return the Data associated with it.
 * If the Name is not found, NULL is returned. */
static char *getarg(char *Name, struct Argument *Scan)
{
        for (; Scan; Scan = Scan->Next)
                if (!strcmp(Name,Scan->Name)) return Scan->Data;
        return NULL;
}

/* Using this function, you can find out which values the user filled in
 * into your HTML form.
 * NOTE: if the user did not enter data, you may get a NULL return
 * value (and not "" which you would expect). */
char *GetArg(char *Name)
{
        return getarg(Name,ArgForm);
}

/* This function is used to check values which are filled in into the
 * document's URL. */
char *GetURLArg(char *Name)
{
        return getarg(Name,ArgURL);
}

/* hex digit conversion */
static int Hex(char c)
{
        if (c>='0' && c<='9') return c-'0';
        if (c>='a' && c<='f') return c-'a'+10;
        if (c>='A' && c<='F') return c-'A'+10;
}

/* HTTP encodes all nonstandard ASCII in the following way:
 * + --> space
 * %xx -> ASCII hex xx
 * This function performs an in-place translation of this data.
 */
static void translate(char *str)
{
        register char *write = str;

        while (*str) {
                if (*str == '+') {
                        *write++ = ' ';
                        str++;
                } else if (*str == '%') {
                        register unsigned char OutChar;

                        str++;
                        OutChar = Hex(*str++)<<4;
                        OutChar += Hex(*str++);
                        *write++ = OutChar;
                } else *write++ = *str++;
        }
        *write = 0;
}

/* This is the workhorse function.
 * Take an input string in the form
 * value1=data1&value2=data2&...
 * and create a linked list containing this data.
 * ArgList is a pointer to the anchor of the list to be created.
 * On the fly, all "value" and "data" strings are converted to ASCII
 * (see translate()) */
static void interpretargs(char *input,struct Argument **ArgList)
{
        char *String;

        *ArgList = NULL;

        /* input MAY be non modifiable (e.g. the result of getenv()).
         * Create a buffer large enough to work on. */
        String = malloc(strlen(input) + 1);
        if (!String) return;

        while (*input) {
                struct Argument *NewArg = malloc(sizeof(struct Argument));
                char *CurrentChar = String;

                if (!NewArg) break;

                /* Get the Name, terminated by = */
                while (*input) {
                        if (*input == '=') {
                                input++; break;
                        }
                        *CurrentChar++ = *input++;
                }
                *CurrentChar = 0;
                translate(String);
                NewArg->Name = malloc(strlen(String)+1);
                strcpy(NewArg->Name,String);

                /* Get the Data, terminated by EOF or & */
                if (*input) {
                        CurrentChar = String;
                        while (*input) {
                                if (*input == '&') {
                                        input++; break;
                                }
                                *CurrentChar++ = *input++;
                        }
                        *CurrentChar = 0;
                        translate(String);
                        NewArg->Data = malloc(strlen(String)+1);
                        strcpy(NewArg->Data,String);
                } else NewArg->Data = NULL;

                NewArg->Next = *ArgList;
                *ArgList = NewArg;
        }
        free(String);
}

/* Fetch the form arguments from stdin and put them in the linked list
 * named ArgForm.
 * The environment variable CONTENT_LENGTH contains the exact
 * number of bytes we are allowed to read from stdin. */
static void GetStdin(void)
{       char *CLen = getenv("CONTENT_LENGTH");
        char *Buffer;
        int Length;

        if (!CLen) return;
        Length = atoi(CLen);
        Buffer = malloc(Length+1);        /* Allow for a null terminator */
        if (!Buffer) return;
        {
                char *Writer = Buffer;

                while (Length--) *Writer++ = fgetc(stdin);
                *Writer = 0;        /* Null-terminate the string */
        }
        interpretargs(Buffer,&ArgForm);
        free(Buffer);
}

/* Fetch the URL arguments from the environment and put them
 * in the linked list named ArgURL. */
static void GetURL(void)
{
        char *String = getenv("QUERY_STRING");

        if (String) interpretargs(String,&ArgURL);
}

/* Your program MUST call this function in order to use
 * GetArg() and GetURLArg() since this function gets the
 * form arguments from stdin, fetches the URL arguments from the
 * environment, and creates two nice linked lists containing them.
 */
void InterpretArgs(void)
{
        GetStdin();
        GetURL();
}
