/*	Mungwall V8-Nov-2000
 *	Copyright (C) Geert Vernaeve (geert.vernaeve@rug.ac.be).
 *	This package catches all memory calls and adds a wall of
 *	bytes in front and at the back of each piece of allocated memory.
 *	The function checkmem() checks to see if the program has thrashed
 *	these walls. All memory allocation/deallocation functions
 *	first call checkmem() to check the state of the memory.
 *	NOTE: if your program allocates and frees a lot, it may become
 *	*VERY* slow.
 *	If NDEBUG is #defined, the package disables itself.
 *	History: 21-Mar-1998	First version
 *		  8-Nov-2000	Edition for the C course
 */

#ifndef NDEBUG

#include <malloc.h>
void mung_checkmem(int,char *,int);	/* Call this to do sanity checks on all allocated memory */
void *mung_malloc(size_t,int,char *);
void *mung_realloc(void *,size_t,int,char *);
void *mung_calloc(size_t,size_t,int,char *);
void mung_free(void *,int,char *);
#ifndef MUNGPRIVATE
#	define malloc(s) mung_malloc(s,__LINE__,__FILE__)
#	define realloc(m,s) mung_realloc(m,s,__LINE__,__FILE__)
#	define calloc(s,t) mung_calloc(s,t,__LINE__,__FILE__)
#	define free(m) mung_free(m,__LINE__,__FILE__)
#	define checkmem() mung_checkmem(__LINE__,__FILE__,1)
#endif

#else

#define checkmem()

#endif
