#include <stdio.h>
#include <fcntl.h>
#include <termio.h>
#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "toets.h"

static struct termio tbufsave;
static int flags;

void initkey(void)
{
        struct termio tbuf;

        /* Als we een letter lezen, en er is er geen beschikbaar
         * (omdat de gebruiker nog niet heeft getikt), dan
         * wacht read() als O_NDELAY uit staat, of geeft 0
         * terug als O_NDELAY aan staat */
        flags=fcntl(0,F_GETFL,0);
        fcntl(0,F_SETFL,flags | O_NDELAY);        /* O_NDELAY aanzetten */

        /* Stel de buffergrootte in op 1 byte. */
        ioctl(0,TCGETA,&tbuf);                    /* Instellingen lezen */
        tbufsave=tbuf;
        /* if you want the key echoed, just remove |ECHO on next line */
        tbuf.c_lflag &= ~(ICANON|ECHO/*|ISIG*/);  /* wis ICANON,ECHO,... */
        tbuf.c_cc[VMIN] = 1;      /* Geef bytes van zodra er 1 klaar staat  */
        ioctl(0,TCSETAF,&tbuf);                   /* Instellingen instellen */
}

void deinitkey(void)
{
        ioctl(0,TCSETA,&tbufsave);
        fcntl(0,F_SETFL,flags);
}

/* Lees een letter, of geef -2 terug als er geen beschikbaar was */
int readkey(void)
{
        static char buf = 0;

        switch(read(0,&buf,1)) {
                case -1: return -1;       /* ERROR */
                case 0:  return -2;        /* No key */
                default: return buf;
        }
}

/* Lees een letter. Blijf desnoods wachten tot er een komt opdagen. */
int mustreadkey(void)
{
        int key;

        fcntl(0,F_SETFL,fcntl(0,F_GETFL,0) && ~O_NDELAY); /* O_NDELAY uitzetten */
        key=readkey() ;                                   /* Wait for key */
        fcntl(0,F_SETFL,fcntl(0,F_GETFL,0) | O_NDELAY);   /* O_NDELAY aanzetten */
        return key;
}
