#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <ctype.h>

#include "gethttp.h"

#define HTTPHEAD "Content-type: text/html\n"

struct Woord {
	struct Woord *next;
	char *van;
	char *naar;
} *woordenlijst = NULL;

char *scriptname;
char *hostname = NULL;

struct tag {
	struct tag *next;
	char *tag;
	char *value;
};

struct Woord *leeswoorden(void) {
	struct Woord *result = NULL;
	char lijn[255];
	FILE *wdb = fopen("woordenlijst","r");
	if (!wdb) return NULL;
	while(fgets(lijn,sizeof(lijn),wdb)) {
		struct Woord *nieuw = malloc(sizeof(struct Woord));
		char *split = strstr(lijn,"\t");
		char *end = lijn + strlen(lijn)-1;	/* points to \n */
		if (!split) continue;
		nieuw->next = result;
		result = nieuw;
		nieuw->van = malloc(split-lijn+1);
		strncpy(nieuw->van,lijn,split-lijn);
		nieuw->van[split-lijn] = 0;
		split++;
		nieuw->naar = malloc(end-split+1);
		strncpy(nieuw->naar,split,end-split);
		nieuw->naar[end-split] = 0;
	}
	fclose(wdb);
	return result;
}

#define printtaglist(t) fprinttaglist(stdout,t)

/* Druk een taglist af, zonder omsluitende '<'...'>' */
void fprinttaglist(FILE *f,struct tag *taglist) {
	while (taglist) {
		if (taglist->value)
			fprintf(f,"%s=\"%s\"",taglist->tag,taglist->value);
		else fprintf(f,"%s",taglist->tag);
		taglist = taglist->next;
		if (taglist) fputc(' ',f);
	}
}

void freetaglist(struct tag *taglist) {
	while (taglist) {
		struct tag *next = taglist->next;
		free(taglist);
		taglist = next;
	}
}

/* *scan wijst naar de tweede letter van een "<blabla ..." string;
 * maak een taglist en geef als resultaat een pointer naar de char
 * vlak na de afsluitende '>' van deze tag */
char *htmlparse(char *scan,struct tag **taglist) {
	struct tag *lasttag = NULL;

	while (1) {
		char *start,*stop;
		struct tag *nieuwetag;
		while (isspace(*scan)) scan++;
		if (*scan == '>') break;
		nieuwetag = malloc(sizeof(struct tag));
		/* tag name */
		start = stop = scan;
		while (*stop && !isspace(*stop) && *stop!='>' && *stop!='=')
			stop++;
		/* tag is opgeslagen in start...stop[-1] */
		nieuwetag->tag = malloc(stop-start+1);
		strncpy(nieuwetag->tag,start,stop-start);
		nieuwetag->tag[stop-start] = 0;
		/* tag value */
		scan = stop;	
		while (isspace(*scan)) scan++;
		if (*scan == '=') {
			/* value */
			scan++;		/* skip '=' */
			while (isspace(*scan)) scan++;
			if (*scan == '"') {
				start = stop = scan+1;
				while (*stop && *stop!='"') stop++;
				scan = stop + 1;
			} else {
				start = stop = scan;
				while (*stop && *stop!='>' && !isspace(*stop))
					stop++;
				scan = stop;
			}
			nieuwetag->value = malloc(stop-start+1);
			strncpy(nieuwetag->value,start,stop-start);
			nieuwetag->value[stop-start] = 0;
		} else nieuwetag->value = NULL;
		/* nieuwetag linken aan de lijst */
		if (lasttag) {
			lasttag->next = nieuwetag;
		} else {
			*taglist = nieuwetag;
		}
		lasttag = nieuwetag;
	}
	if (lasttag) lasttag->next = NULL;
	if (*scan == '>') scan++;
	return scan;
}

/* s is een url; laat doorverwijzen naar dit script */
void printthrow(char *s) {
	if (*s == '/') {
		printf("%s/%s%s",scriptname,hostname,s);
	} else if (!strncasecmp("http://",s,7)) {
		printf("%s%s",scriptname,s+6);
	} else printf("%s",s);
}

/* *scan wijst naar een "<blabla ..." string; verwerk deze tag
 * en geef als resultaat een pointer naar de char vlak na de afsluitende
 * '>' van deze tag */
char *dotag(char *scan) {
	putchar(*scan++);	/* skip '<' */
	while (isspace(*scan)) putchar(*scan++);


	if (!strncasecmp(scan,"a",1)) {
		struct tag *taglist,*snuffel;
		scan = htmlparse(scan,&taglist);
		for (snuffel = taglist; snuffel; snuffel = snuffel->next) {
			if (!strcasecmp(snuffel->tag,"href") && snuffel->value) {
				printf("%s=\"",snuffel->tag);
				printthrow(snuffel->value);
				putchar('"');
			} else if (snuffel->value)
				printf("%s=\"%s\"",snuffel->tag,snuffel->value);
			else printf("%s",snuffel->tag);
			if (snuffel->next) putchar(' ');
		}
		freetaglist(taglist);
		putchar('>');
	} else if (!strncasecmp(scan,"img",3)) {
		struct tag *taglist,*snuffel;
		scan = htmlparse(scan,&taglist);
		for (snuffel = taglist; snuffel; snuffel = snuffel->next) {
			if (!strcasecmp(snuffel->tag,"src") && snuffel->value) {
				printf("%s=\"",snuffel->tag);
				printthrow(snuffel->value);
				putchar('"');
			} else if (snuffel->value)
				printf("%s=\"%s\"",snuffel->tag,snuffel->value);
			else printf("%s",snuffel->tag);
			if (snuffel->next) putchar(' ');
		}
		freetaglist(taglist);
		putchar('>');
	} else if (!strncasecmp(scan,"body",4)) {
		struct tag *taglist,*snuffel;
		scan = htmlparse(scan,&taglist);
		for (snuffel = taglist; snuffel; snuffel = snuffel->next) {
			if (!strcasecmp(snuffel->tag,"background") && snuffel->value) {
				printf("%s=\"",snuffel->tag);
				printthrow(snuffel->value);
				putchar('"');
			} else if (snuffel->value)
				printf("%s=\"%s\"",snuffel->tag,snuffel->value);
			else printf("%s",snuffel->tag);
			if (snuffel->next) putchar(' ');
		}
		freetaglist(taglist);
		putchar('>');
	} else {		/* default */
		while (*scan) {
			putchar(*scan++);
			if (*scan == '>') return scan;
		}
	}
	return scan;
}

/* *woord wijst naar het begin van een woord
 * return value: moet vlak na het woord wijzen */
char *doewoord(char *woord) {
	char *eind;
	struct Woord *snuffel;
	int lengte;

	for (eind = woord; isalpha(*eind); eind++) ;
	lengte = eind-woord;

	/* vertaal */
	for (snuffel = woordenlijst; snuffel; snuffel = snuffel->next) {
		if (strlen(snuffel->van) != lengte) continue;
		if (!strncasecmp(snuffel->van,woord,lengte)) {
			/* hoe staat het originele woord er? */
			if (islower(*woord)) {	/* kleine letters */
				printf("%s",snuffel->naar);
			} else if (isupper(woord[1])) {	/* HOOFDLETTERS */
				char *walk;
				for (walk = snuffel->naar; *walk; walk++)
					putchar(toupper(*walk));
			} else {	/* Gekapitaliseerd */
				printf("%c%s",toupper(snuffel->naar[0]),snuffel->naar+1);
			}
			return eind;
		}
	}

	/* geen vertaling ... */
	{	char *walk;
		for (walk = woord; walk < eind; walk++)
			putchar(*walk);
	}
	return eind;
}

int main(int argc,char **argv) {
	char *path = getenv("PATH_INFO");
	char *fileonhost;
	struct httpDocument *doc;
	int html = 0;

	if (!path) {
		puts(HTTPHEAD "\n<H1>Fout</H1>Geen PATH_INFO gevonden");
		return 10;
	}

	/* Usage: http://.../cgi-bin/vertaal/hostname/path/page */
	{	char *scan;
		for (scan = path+1; *scan; scan++) {
			if (*scan == '/') {
				hostname = malloc(scan-path);
				strncpy(hostname,path+1,scan-path-1);
				hostname[scan-path-1] = 0;
				fileonhost = scan;
				break;
			}
		}
	}

	if (!hostname) {	/* bv: .../vertaal/www.rug.ac.be */
		hostname = path+1;
		fileonhost = "/";
	}

	doc = gethttp(gethostbyname(hostname),fileonhost,NULL);

	if (!doc) {
		puts(HTTPHEAD "\n<H1>Fout bij het ophalen van het document");
		return 12;
	}

	/** content-type bepalen */
	{	char *type = strstr(doc->header,"Content-Type");

		if (!type) puts("Content-Type: text/html\n");
		else {
			if (!strncmp(type+14,"text/html",9)) html = 1;
			while (*type!='\n' && *type!='\r' && *type)
				putchar(*type++);
			puts("\n");
		}
	}

	if (html) {
		/* Alle links naar /... moeten aangepast worden */
		char *scan = doc->body;
		woordenlijst = leeswoorden();
		scriptname = getenv("SCRIPT_NAME");
		if (!scriptname) scriptname = "";	/* tsja */
		while (*scan) {
			if (*scan == '<')
				scan = dotag(scan);
			else if (isalpha(*scan))
				scan = doewoord(scan);
			else putchar(*scan++);
		}
	} else {
		fwrite(doc->body,doc->bodyLength,1,stdout);
	}

	return 0;
}
